#include "gelement.h"
#include <string.h>

gelement::gelement (void)
{
  //  number of nodes on element
  nne = 0;
  //  number of degrees of freedom
  ndofe = 0;
  //  indicator of code numbers on element (code numbers are on nodes)
  cne = 0;
  //  type of element (with respect to geometry)
  get=noelem;
  //  number of additional degrees of freedom (especially Lagrange multipliers)
  nmult=0;

  //  array of node numbers
  nodes=NULL;
  auxinf=0;
  
  //edgn = NULL;
  //  array for code numbers, if necessary
  cn = NULL;
  //  general time function
  tgf=0;
}

gelement::~gelement (void)
{
  delete [] nodes;
  //delete [] edgn;
  delete [] cn;
}

/**
   function reads informations from input file
   
   @param in - input stream
   @param m - number nodes on element
   @param n - number of degrees of freedom on element
   @param et - type of element
   
   16.7.2002
*/
void gelement::read (XFILE *in,long m,long n,gelemtype et)
{
  long i;
  
  //  type of finite element with respect to geometry
  get=et;
  //  number nodes on element
  nne=m;
  //  number of degrees of freedom on element
  ndofe=n;
  
  nodes = new long [nne];
  for (i=0;i<nne;i++){
    xfscanf (in,"%ld",nodes+i);
    nodes[i]--;
  }
  
  xfscanf (in,"%ld",&cne);
  if (cne!=0 && cne!=1){
    fprintf (stderr,"\n\n identification of code numbers on element (cne) must be equal");
    fprintf (stderr,"\n to zero or one in function gelement::read (%s, line %d).\n",__FILE__,__LINE__);
  }
  
  // spring element type
  if ((ndofe < -1) && (nne == 1))
    return;

  if (cne==1){
    cn = new long [ndofe];
    for (i=0;i<ndofe;i++){
      xfscanf (in,"%ld",cn+i);
    }
  }
  
}

/**
   function reads informations from input file
   
   @param in - input stream
   @param m - number nodes on element
   @param n - number of degrees of freedom on element
   
   16.7.2002
*/
void gelement::read_gf (XFILE *in,long m,long n)
{
  long i;

  //  number nodes on element
  nne=m;
  //  number of degrees of freedom on element
  ndofe=n;
  
  nodes = new long [nne];
  for (i=0;i<nne;i++){
    xfscanf (in,"%ld",nodes+i);
    nodes[i]--;
  }
  
  xfscanf (in,"%ld",&cne);
  if (cne!=0 && cne!=1){
    fprintf (stderr,"\n\n identification of code numbers on element (cne) must be equal");
    fprintf (stderr,"\n to zero or one in function gelement::read (%s, line %d).\n",__FILE__,__LINE__);
  }
  
  // spring element type
  if ((ndofe < -1) && (nne == 1))
    return;

  if (cne==1){
    cn = new long [ndofe];
    for (i=0;i<ndofe;i++){
      xfscanf (in,"%ld",cn+i);
    }
  }
  
  xfscanf (in,"%ld",&tgf);
  tgf--;
}


/**
   function initiates element values
   function is used in parallel computation
   
   @param icn - array containing element code numbers
   @param m - number of degrees of freedom on element
   
   24.7.2002
*/
void gelement::initiate (long *icn,long m)
{
  long i;
  cne=1;
  ndofe=m;
  
  cn = new long [ndofe];
  for (i=0;i<ndofe;i++){
    cn[i]=icn[i];
  }
  
}

/**
   function returns number of nodes on element
   
   JK
*/
long gelement::give_nne ()
{
  return nne;
}

/**
   function returns number of degrees of freedom of element
   
   JK
*/
long gelement::give_ndofe ()
{
  return ndofe;
}

/**
   function returns number of Lagrange multipliers defined on element
   
   JK
*/
long gelement::give_nmult ()
{
  return nmult;
}

/**
   function selects nodes of element
   
   @param nod - vector containing selected nodes
   
   JK
*/
void gelement::give_nodes (ivector &nod)
{
  long i;
  if (nod.n!=nne){
    fprintf (stderr,"\n\n wrong size of array is assumed in function give_nodes (%s, line %d).\n",__FILE__,__LINE__);
  }

  for (i=0;i<nne;i++){
    nod[i]=nodes[i];
  }
}

/**
   function returns code number indicator of element
   if cne=1, code numbers are defined on element,
   otherwise code numbers are defined on nodes
   
   JK
*/
long gelement::give_cne ()
{
  return cne;
}

